import { MatterJsPhysics } from "../physics/matterjs_physics";
import { SvgData } from "../base/svg_data";
import { expandCommands } from "../base/command_mapper";
import { makeAbsolute, parseSVG } from "svg-path-parser";
import { GravityPlugin } from "../physics/plugins/gravity_plugin";
import { ViewColliderBorder } from "../physics/plugins/view_collider_border_plugin";
import { MousePlugin } from "../physics/plugins/mouse_plugin";
import { RendererPlugin } from "../physics/plugins/renderer_plugin";
import { ProductionRenderer } from "../renderer/production_renderer";
import { ScrollPlugin } from "../physics/plugins/scroll_plugin";
import { LimitVelocityPlugin } from "../physics/plugins/limit_velocity_plugin";
import { LimitAngularVelocityPlugin } from "../physics/plugins/limit_angular_velocity_plugin";
import { StopPhysicalSvg } from "./stop_builder";
var SimulationBuilder = /** @class */ (function () {
    function SimulationBuilder(aggregator, svg) {
        this._aggregator = aggregator;
        this._svg = svg;
        this._plugins = [];
    }
    SimulationBuilder.prototype.withGravity = function (gravity) {
        this._plugins.push(new GravityPlugin(gravity));
        return this;
    };
    SimulationBuilder.prototype.withBorderCollider = function () {
        this._plugins.push(new ViewColliderBorder());
        return this;
    };
    SimulationBuilder.prototype.withMouseInteraction = function (mouseForce) {
        this._plugins.push(new MousePlugin(mouseForce));
        return this;
    };
    SimulationBuilder.prototype.withScrollInteraction = function (scrollForce) {
        this._plugins.push(new ScrollPlugin(scrollForce));
        return this;
    };
    SimulationBuilder.prototype.withDebugRenderer = function (elementToAddCanvas) {
        this._plugins.push(new RendererPlugin(elementToAddCanvas));
        return this;
    };
    SimulationBuilder.prototype.withCustomPlugin = function (plugin) {
        this._plugins.push(plugin);
        return this;
    };
    SimulationBuilder.prototype.withLimitVelocityPlugin = function (maxSpeed) {
        this._plugins.push(new LimitVelocityPlugin(maxSpeed));
        return this;
    };
    SimulationBuilder.prototype.withLimitAngularVelocityPlugin = function (maxSpeed) {
        this._plugins.push(new LimitAngularVelocityPlugin(maxSpeed));
        return this;
    };
    SimulationBuilder.prototype.startSimulation = function (config) {
        var svgData = this.createSvgData();
        var physics = new MatterJsPhysics(svgData, config, this._plugins);
        var renderer = new ProductionRenderer();
        renderer.renderLoop(svgData, physics);
        return new StopPhysicalSvg(physics, renderer);
    };
    SimulationBuilder.prototype.createSvgData = function () {
        var paths = Array.prototype.slice.call(this._svg.getElementsByTagName('path'));
        var pathCommandArray = paths
            .map(function (path) { return path.getAttribute("d"); })
            .map(function (pathString) { return expandCommands(makeAbsolute(parseSVG(pathString))); });
        var aggregatedCommands = this._aggregator.aggregate(pathCommandArray);
        return new SvgData(this._svg, pathCommandArray, aggregatedCommands);
    };
    return SimulationBuilder;
}());
export { SimulationBuilder };
//# sourceMappingURL=simulation_builder.js.map